@@ -1,28 +1,16 @@
-=head1 NAME
-
-Petal::Cache::Memory - Caches generated subroutines in memory.
-
-=head1 SYNOPSIS
-
-  use Petal::Cache::Memory;
-  my $coderef = Petal::Cache::Memory->get ('foo.html');
-  unless (defined $coderef)
-  {
-    $coderef = complicated_long_compute_thing();
-    Petal::Cache::Memory->set ($coderef);
-  }
-
-=head1 DESCRIPTION
-
-  A simple cache module to avoid re-compiling the Perl
-  code from the Perl data at each request
-
-=cut
+# ------------------------------------------------------------------
+# Petal::Cache::Memory - Caches generated subroutines in memory.
+# ------------------------------------------------------------------
+# Author: Jean-Michel Hiver
+# Description: A simple cache module to avoid re-compiling the Perl
+# code from the Perl data at each request.
+# ------------------------------------------------------------------
 package Petal::Cache::Memory;
 use strict;
 use warnings;
 use Carp;
 
+
 our $FILE_TO_SUBS  = {};
 our $FILE_TO_MTIME = {};
 
@@ -33,17 +21,11 @@ sub sillyness
 }
 
 
-=head1 METHODS
-
-All the methods are static methods.
-
-=head2 $class->get ($file);
-
-Returns the cached subroutine if its last modification time
-is more recent than the last modification time of the template,
-returns undef otherwise
-
-=cut
+# $class->get ($file);
+# --------------------
+# Returns the cached subroutine if its last modification time
+# is more recent than the last modification time of the template,
+# returns undef otherwise
 sub get
 {
     my $class = shift;
@@ -55,11 +37,9 @@ sub get
 }
 
 
-=head2 $class->set ($file, $code);
-
-Sets the cached code for $file.
-
-=cut
+# $class->set ($file, $code);
+# ---------------------------
+# Sets the cached code for $file.
 sub set
 {
     my $class = shift;
@@ -71,11 +51,9 @@ sub set
 }
 
 
-=head2 $class->is_ok ($file);
-
-Returns TRUE if the cache is still fresh, FALSE otherwise.
-
-=cut
+# $class->is_ok ($file);
+# ----------------------
+# Returns TRUE if the cache is still fresh, FALSE otherwise.
 sub is_ok
 {
     my $class = shift;
@@ -89,12 +67,10 @@ sub is_ok
 }
 
 
-=head2 $class->cached_mtime ($file);
-
-Returns the last modification date of the cached data
-for $file
-
-=cut
+# $class->cached_mtime ($file);
+# -----------------------------
+# Returns the last modification date of the cached data
+# for $file
 sub cached_mtime
 {
     my $class = shift;
@@ -104,11 +80,9 @@ sub cached_mtime
 }
 
 
-=head2 $class->current_mtime ($file);
-
-Returns the last modification date for $file
-
-=cut
+# $class->current_mtime ($file);
+# ------------------------------
+# Returns the last modification date for $file
 sub current_mtime
 {
     my $class = shift;
@@ -118,13 +92,11 @@ sub current_mtime
 }
 
 
-=head2 $class->compute_key ($file);
-
-Computes a cache 'key' for $file, which should be unique.
-(Well, currently an MD5 checksum is used, which is not
-*exactly* unique but which should be good enough)
-
-=cut
+# $class->compute_key ($file);
+# ----------------------------
+# Computes a cache 'key' for $file, which should be unique.
+# (Well, currently an MD5 checksum is used, which is not
+# *exactly* unique but which should be good enough)
 sub compute_key
 {
     my $class = shift;
@@ -136,15 +108,3 @@ sub compute_key
 
 
 1;
-
-
-__END__
-
-
-=head1 AUTHOR
-
-Jean-Michel Hiver <jhiver@mkdoc.com>
-
-This module is redistributed under the same license as Perl itself.
-
-=cut
